#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
migration interface
"""

from novaclient import api_versions
from novaclient import base


class Migration(base.Resource):
    def __repr__(self):
        return "<Migration: %s>" % self.id


class MigrationManager(base.ManagerWithFind):
    resource_class = Migration

    def _list_base(self, host=None, status=None, instance_uuid=None,
                   marker=None, limit=None, changes_since=None,
                   changes_before=None, migration_type=None,
                   source_compute=None, user_id=None, project_id=None):
        opts = {}
        if host:
            opts['host'] = host
        if status:
            opts['status'] = status
        if instance_uuid:
            opts['instance_uuid'] = instance_uuid
        if marker:
            opts['marker'] = marker
        if limit:
            opts['limit'] = limit
        if changes_since:
            opts['changes-since'] = changes_since
        if changes_before:
            opts['changes-before'] = changes_before
        if migration_type:
            opts['migration_type'] = migration_type
        if source_compute:
            opts['source_compute'] = source_compute
        if user_id:
            opts['user_id'] = user_id
        if project_id:
            opts['project_id'] = project_id

        return self._list("/os-migrations", "migrations", filters=opts)

    @api_versions.wraps("2.0", "2.58")
    def list(self, host=None, status=None, instance_uuid=None,
             migration_type=None, source_compute=None):
        """
        Get a list of migrations.
        :param host: filter migrations by host name (optional).
        :param status: filter migrations by status (optional).
        :param instance_uuid: filter migrations by instance uuid (optional).
        :param migration_type: Filter migrations by type. Valid values are:
        evacuation, live-migration, migration (cold), resize
        :param source_compute: Filter migrations by source compute host name.
        """
        return self._list_base(host=host, status=status,
                               instance_uuid=instance_uuid,
                               migration_type=migration_type,
                               source_compute=source_compute)

    @api_versions.wraps("2.59", "2.65")
    def list(self, host=None, status=None, instance_uuid=None,
             marker=None, limit=None, changes_since=None,
             migration_type=None, source_compute=None):
        """
        Get a list of migrations.
        :param host: filter migrations by host name (optional).
        :param status: filter migrations by status (optional).
        :param instance_uuid: filter migrations by instance uuid (optional).
        :param marker: Begin returning migrations that appear later in the
        migrations list than that represented by this migration UUID
        (optional).
        :param limit: maximum number of migrations to return (optional).
        Note the API server has a configurable default limit. If no limit is
        specified here or limit is larger than default, the default limit will
        be used.
        :param changes_since: only return migrations changed later or equal
        to a certain point of time. The provided time should be an ISO 8061
        formatted time. e.g. 2016-03-04T06:27:59Z . (optional).
        :param migration_type: Filter migrations by type. Valid values are:
        evacuation, live-migration, migration (cold), resize
        :param source_compute: Filter migrations by source compute host name.
        """
        return self._list_base(host=host, status=status,
                               instance_uuid=instance_uuid,
                               marker=marker, limit=limit,
                               changes_since=changes_since,
                               migration_type=migration_type,
                               source_compute=source_compute)

    @api_versions.wraps("2.66", "2.79")
    def list(self, host=None, status=None, instance_uuid=None,
             marker=None, limit=None, changes_since=None,
             changes_before=None, migration_type=None, source_compute=None):
        """
        Get a list of migrations.
        :param host: filter migrations by host name (optional).
        :param status: filter migrations by status (optional).
        :param instance_uuid: filter migrations by instance uuid (optional).
        :param marker: Begin returning migrations that appear later in the
        migrations list than that represented by this migration UUID
        (optional).
        :param limit: maximum number of migrations to return (optional).
        Note the API server has a configurable default limit. If no limit is
        specified here or limit is larger than default, the default limit will
        be used.
        :param changes_since: Only return migrations changed later or equal
        to a certain point of time. The provided time should be an ISO 8061
        formatted time. e.g. 2016-03-04T06:27:59Z . (optional).
        :param changes_before: Only return migrations changed earlier or
        equal to a certain point of time. The provided time should be an ISO
        8061 formatted time. e.g. 2016-03-05T06:27:59Z . (optional).
        :param migration_type: Filter migrations by type. Valid values are:
        evacuation, live-migration, migration (cold), resize
        :param source_compute: Filter migrations by source compute host name.
        """
        return self._list_base(host=host, status=status,
                               instance_uuid=instance_uuid,
                               marker=marker, limit=limit,
                               changes_since=changes_since,
                               changes_before=changes_before,
                               migration_type=migration_type,
                               source_compute=source_compute)

    @api_versions.wraps("2.80")
    def list(self, host=None, status=None, instance_uuid=None,
             marker=None, limit=None, changes_since=None,
             changes_before=None, migration_type=None,
             source_compute=None, user_id=None, project_id=None):
        """
        Get a list of migrations.
        :param host: filter migrations by host name (optional).
        :param status: filter migrations by status (optional).
        :param instance_uuid: filter migrations by instance uuid (optional).
        :param marker: Begin returning migrations that appear later in the
        migrations list than that represented by this migration UUID
        (optional).
        :param limit: maximum number of migrations to return (optional).
        Note the API server has a configurable default limit. If no limit is
        specified here or limit is larger than default, the default limit will
        be used.
        :param changes_since: Only return migrations changed later or equal
        to a certain point of time. The provided time should be an ISO 8061
        formatted time. e.g. 2016-03-04T06:27:59Z . (optional).
        :param changes_before: Only return migrations changed earlier or
        equal to a certain point of time. The provided time should be an ISO
        8061 formatted time. e.g. 2016-03-05T06:27:59Z . (optional).
        :param migration_type: Filter migrations by type. Valid values are:
        evacuation, live-migration, migration, resize
        :param source_compute: Filter migrations by source compute host name.
        :param user_id: filter migrations by user (optional).
        :param project_id: filter migrations by project (optional).
        """
        return self._list_base(host=host, status=status,
                               instance_uuid=instance_uuid,
                               marker=marker, limit=limit,
                               changes_since=changes_since,
                               changes_before=changes_before,
                               migration_type=migration_type,
                               source_compute=source_compute,
                               user_id=user_id,
                               project_id=project_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from tempest.lib import exceptions

from novaclient.tests.functional import base


class SimpleReadOnlyNovaClientTest(base.ClientTestBase):

    """Read only functional python-novaclient tests.

    This only exercises client commands that are read only.
    """

    COMPUTE_API_VERSION = "2.1"

    def test_admin_fake_action(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.nova,
                          'this-does-nova-exist')

    # NOTE(jogo): Commands in order listed in 'nova help'

    def test_admin_aggregate_list(self):
        self.nova('aggregate-list')

    def test_admin_availability_zone_list(self):
        self.assertIn("internal", self.nova('availability-zone-list'))

    def test_admin_flavor_access_list(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.nova,
                          'flavor-access-list')
        # Failed to get access list for public flavor type
        self.assertRaises(exceptions.CommandFailed,
                          self.nova,
                          'flavor-access-list',
                          params='--flavor m1.tiny')

    def test_admin_flavor_list(self):
        self.assertIn("Memory_MiB", self.nova('flavor-list'))

    def test_admin_hypervisor_list(self):
        self.nova('hypervisor-list')

    @decorators.skip_because(bug="1157349")
    def test_admin_interface_list(self):
        self.nova('interface-list')

    def test_admin_keypair_list(self):
        self.nova('keypair-list')

    def test_admin_list(self):
        self.nova('list')
        self.nova('list', params='--all-tenants 1')
        self.nova('list', params='--all-tenants 0')
        self.assertRaises(exceptions.CommandFailed,
                          self.nova,
                          'list',
                          params='--all-tenants bad')

    def test_admin_server_group_list(self):
        self.nova('server-group-list')

    def test_admin_service_list(self):
        self.nova('service-list')

    def test_admin_usage(self):
        self.nova('usage')

    def test_admin_usage_list(self):
        self.nova('usage-list')

    def test_admin_help(self):
        self.nova('help')

    def test_agent_list(self):
        ex = self.assertRaises(exceptions.CommandFailed,
                               self.nova, 'agent-list')
        self.assertIn(
            "This resource is no longer available. "
            "No forwarding address is given. (HTTP 410)", str(ex))
        self.assertIn(
            "This command has been deprecated since 23.0.0 Wallaby Release "
            "and will be removed in the first major release "
            "after the Nova server 24.0.0 X release.", str(ex.stderr))
        ex = self.assertRaises(exceptions.CommandFailed,
                               self.nova, 'agent-list', flags='--debug')
        self.assertIn(
            "This resource is no longer available. "
            "No forwarding address is given. (HTTP 410)", str(ex))
        self.assertIn(
            "This command has been deprecated since 23.0.0 Wallaby Release "
            "and will be removed in the first major release "
            "after the Nova server 24.0.0 X release.", str(ex.stderr))

    def test_migration_list(self):
        self.nova('migration-list')
        self.nova('migration-list', flags='--debug')

    def test_version_list(self):
        self.nova('version-list', flags='--debug')

    def test_quota_defaults(self):
        self.nova('quota-defaults')
        self.nova('quota-defaults', flags='--debug')

    def test_bash_completion(self):
        self.nova('bash-completion')

    # Optional arguments:

    def test_admin_version(self):
        self.nova('', flags='--version')

    def test_admin_debug_list(self):
        self.nova('list', flags='--debug')

    def test_admin_timeout(self):
        self.nova('list', flags='--timeout %d' % 60)

    def test_admin_timing(self):
        self.nova('list', flags='--timing')

    def test_admin_invalid_bypass_url(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.nova,
                          'list',
                          flags='--os-endpoint-override badurl')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base
from novaclient.tests.functional.v2.legacy import test_os_services
from novaclient import utils


class TestOsServicesNovaClientV211(test_os_services.TestOsServicesNovaClient):
    """Functional tests for os-services attributes, microversion 2.11"""

    COMPUTE_API_VERSION = "2.11"

    def test_os_services_force_down_force_up(self):
        for serv in self.client.services.list():
            # In Pike the os-services API was made multi-cell aware and it
            # looks up services by host, which uses the host mapping record
            # in the API DB which is only populated for nova-compute services,
            # effectively making it impossible to perform actions like enable
            # or disable non-nova-compute services since the API won't be able
            # to find them. So filter out anything that's not nova-compute.
            if serv.binary != 'nova-compute':
                continue
            service_list = self.nova('service-list --binary %s' % serv.binary)
            # Check the 'service-list' table has the 'Forced down' column
            status = self._get_column_value_from_single_row_table(
                service_list, 'Forced down')
            self.assertEqual('False', status)

            host = self._get_column_value_from_single_row_table(service_list,
                                                                'Host')
            service = self.nova('service-force-down %s' % host)
            self.addCleanup(self.nova, 'service-force-down --unset',
                            params=host)
            status = self._get_column_value_from_single_row_table(
                service, 'Forced down')
            self.assertEqual('True', status)
            service = self.nova('service-force-down --unset %s' % host)
            status = self._get_column_value_from_single_row_table(
                service, 'Forced down')
            self.assertEqual('False', status)


class TestOsServicesNovaClientV2_53(base.ClientTestBase):
    """Tests the nova service-* commands using the 2.53 microversion.

    The main difference with the 2.53 microversion in these commands is
    the host/binary combination is replaced with the service.id as the
    unique identifier for a service.
    """
    COMPUTE_API_VERSION = "2.53"

    def test_os_services_list(self):
        table = self.nova('service-list')
        for serv in self.client.services.list():
            self.assertIn(serv.binary, table)
            # the id should not be an integer and should be in the table
            self.assertFalse(utils.is_integer_like(serv.id))
            self.assertIn(serv.id, table)

    def test_os_service_disable_enable(self):
        # Disable and enable Nova services in accordance with list of nova
        # services returned by client
        # NOTE(sdague): service disable has the chance in racing
        # with other tests. Now functional tests for novaclient are launched
        # in serial way (https://review.opendev.org/#/c/217768/), but
        # it's a potential issue for making these tests parallel in the future
        for serv in self.client.services.list():
            # In Pike the os-services API was made multi-cell aware and it
            # looks up services by host, which uses the host mapping record
            # in the API DB which is only populated for nova-compute services,
            # effectively making it impossible to perform actions like enable
            # or disable non-nova-compute services since the API won't be able
            # to find them. So filter out anything that's not nova-compute.
            if serv.binary != 'nova-compute':
                continue
            service = self.nova('service-disable %s' % serv.id)
            self.addCleanup(self.nova, 'service-enable', params="%s" % serv.id)
            service_id = self._get_column_value_from_single_row_table(
                service, 'ID')
            self.assertEqual(serv.id, service_id)
            status = self._get_column_value_from_single_row_table(
                service, 'Status')
            self.assertEqual('disabled', status)
            service = self.nova('service-enable %s' % serv.id)
            service_id = self._get_column_value_from_single_row_table(
                service, 'ID')
            self.assertEqual(serv.id, service_id)
            status = self._get_column_value_from_single_row_table(
                service, 'Status')
            self.assertEqual('enabled', status)

    def test_os_service_disable_log_reason(self):
        for serv in self.client.services.list():
            # In Pike the os-services API was made multi-cell aware and it
            # looks up services by host, which uses the host mapping record
            # in the API DB which is only populated for nova-compute services,
            # effectively making it impossible to perform actions like enable
            # or disable non-nova-compute services since the API won't be able
            # to find them. So filter out anything that's not nova-compute.
            if serv.binary != 'nova-compute':
                continue
            service = self.nova('service-disable --reason test_disable %s'
                                % serv.id)
            self.addCleanup(self.nova, 'service-enable', params="%s" % serv.id)
            service_id = self._get_column_value_from_single_row_table(
                service, 'ID')
            self.assertEqual(serv.id, service_id)
            status = self._get_column_value_from_single_row_table(
                service, 'Status')
            log_reason = self._get_column_value_from_single_row_table(
                service, 'Disabled Reason')
            self.assertEqual('disabled', status)
            self.assertEqual('test_disable', log_reason)

    def test_os_services_force_down_force_up(self):
        for serv in self.client.services.list():
            # In Pike the os-services API was made multi-cell aware and it
            # looks up services by host, which uses the host mapping record
            # in the API DB which is only populated for nova-compute services,
            # effectively making it impossible to perform actions like enable
            # or disable non-nova-compute services since the API won't be able
            # to find them. So filter out anything that's not nova-compute.
            if serv.binary != 'nova-compute':
                continue
            service = self.nova('service-force-down %s' % serv.id)
            self.addCleanup(self.nova, 'service-force-down --unset',
                            params="%s" % serv.id)
            service_id = self._get_column_value_from_single_row_table(
                service, 'ID')
            self.assertEqual(serv.id, service_id)
            forced_down = self._get_column_value_from_single_row_table(
                service, 'Forced down')
            self.assertEqual('True', forced_down)
            service = self.nova('service-force-down --unset %s' % serv.id)
            forced_down = self._get_column_value_from_single_row_table(
                service, 'Forced down')
            self.assertEqual('False', forced_down)



# Copyright 2018 NTT Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64
import subprocess
from unittest import mock

from novaclient import crypto
from novaclient.tests.unit import utils


class CryptoTest(utils.TestCase):

    def setUp(self):
        super(CryptoTest, self).setUp()
        # The password string that passed as the method argument
        self.password_string = 'Test Password'
        # The return value of Popen.communicate
        self.decrypt_password = b'Decrypt Password'
        self.private_key = 'Test Private Key'

    @mock.patch('subprocess.Popen')
    def test_decrypt_password(self, mock_open):
        mocked_proc = mock.Mock()
        mock_open.return_value = mocked_proc
        mocked_proc.returncode = 0
        mocked_proc.communicate.return_value = (self.decrypt_password, '')

        decrypt_password = crypto.decrypt_password(self.private_key,
                                                   self.password_string)

        # The return value is 'str' in both python 2 and python 3
        self.assertIsInstance(decrypt_password, str)
        self.assertEqual('Decrypt Password', decrypt_password)

        mock_open.assert_called_once_with(
            ['openssl', 'rsautl', '-decrypt', '-inkey', self.private_key],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        mocked_proc.communicate.assert_called_once_with(
            base64.b64decode(self.password_string))
        mocked_proc.stdin.close.assert_called_once_with()

    @mock.patch('subprocess.Popen')
    def test_decrypt_password_failure(self, mock_open):
        mocked_proc = mock.Mock()
        mock_open.return_value = mocked_proc
        mocked_proc.returncode = 1  # Error case
        mocked_proc.communicate.return_value = (self.decrypt_password, '')

        self.assertRaises(crypto.DecryptionFailure, crypto.decrypt_password,
                          self.private_key, self.password_string)

        mock_open.assert_called_once_with(
            ['openssl', 'rsautl', '-decrypt', '-inkey', self.private_key],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        mocked_proc.communicate.assert_called_once_with(
            base64.b64decode(self.password_string))
        mocked_proc.stdin.close.assert_called_once_with()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures
from keystoneauth1 import fixture
from keystoneauth1 import loading
from keystoneauth1 import session

from novaclient import client

IDENTITY_URL = 'http://identity.host/v3'
COMPUTE_URL = 'http://compute.host'


class V1(fixtures.Fixture):

    def __init__(self, requests_mock,
                 compute_url=COMPUTE_URL, identity_url=IDENTITY_URL,
                 **client_kwargs):
        super(V1, self).__init__()
        self.identity_url = identity_url
        self.compute_url = compute_url
        self.client = None
        self.requests_mock = requests_mock

        self.token = fixture.V2Token()
        self.token.set_scope()
        self.discovery = fixture.V2Discovery(href=self.identity_url)

        s = self.token.add_service('compute')
        s.add_endpoint(self.compute_url)

        s = self.token.add_service('computev3')
        s.add_endpoint(self.compute_url)

        self._client_kwargs = client_kwargs

    def setUp(self):
        super(V1, self).setUp()

        auth_url = '%s/tokens' % self.identity_url
        headers = {'X-Content-Type': 'application/json'}
        self.requests_mock.post(auth_url,
                                json=self.token,
                                headers=headers)
        self.requests_mock.get(self.identity_url,
                               json=self.discovery,
                               headers=headers)
        self.client = self.new_client(**self._client_kwargs)

    def new_client(self, **client_kwargs):
        return client.Client("2", username='xx',
                             password='xx',
                             project_id='xx',
                             auth_url=self.identity_url,
                             **client_kwargs)


class SessionV1(V1):

    def new_client(self):
        self.session = session.Session()
        loader = loading.get_plugin_loader('password')
        self.session.auth = loader.load_from_options(
            auth_url=self.identity_url, username='xx', password='xx')
        return client.Client("2", session=self.session)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient.tests.unit import fakes
from novaclient.tests.unit.fixture_data import base


class V1(base.Fixture):

    base_url = 'v2/images'

    def setUp(self):
        super(V1, self).setUp()

        get_images = {
            'images': [
                {'id': 1, 'name': 'CentOS 5.2'},
                {'id': 2, 'name': 'My Server Backup'}
            ]
        }

        headers = self.json_headers

        self.requests_mock.get(self.url(),
                               json=get_images,
                               headers=headers)

        image_1 = {
            'id': 1,
            'name': 'CentOS 5.2',
            "updated": "2010-10-10T12:00:00Z",
            "created": "2010-08-10T12:00:00Z",
            "status": "ACTIVE",
            "metadata": {
                "test_key": "test_value",
            },
            "links": {},
        }

        image_2 = {
            "id": 2,
            "name": "My Server Backup",
            "serverId": 1234,
            "updated": "2010-10-10T12:00:00Z",
            "created": "2010-08-10T12:00:00Z",
            "status": "SAVING",
            "progress": 80,
            "links": {},
        }

        self.requests_mock.get(self.url('detail'),
                               json={'images': [image_1, image_2]},
                               headers=headers)

        self.requests_mock.get(self.url(1),
                               json={'image': image_1},
                               headers=headers)

        def post_images_1_metadata(request, context):
            body = request.json()
            assert list(body) == ['metadata']
            fakes.assert_has_keys(body['metadata'], required=['test_key'])
            return {'metadata': image_1['metadata']}

        self.requests_mock.post(self.url(1, 'metadata'),
                                json=post_images_1_metadata,
                                headers=headers)

        for u in (1, '1/metadata/test_key'):
            self.requests_mock.delete(self.url(u), status_code=204,
                                      headers=headers)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from novaclient.tests.unit.fixture_data import client
from novaclient.tests.unit.fixture_data import images as data
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import images


class ImagesTest(utils.FixturedTestCase):

    client_fixture_class = client.V1
    data_fixture_class = data.V1

    @mock.patch('novaclient.base.Manager.alternate_service_type')
    def test_list_images(self, mock_alternate_service_type):
        il = self.cs.glance.list()
        self.assert_request_id(il, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/v2/images')
        for i in il:
            self.assertIsInstance(i, images.Image)
        self.assertEqual(2, len(il))
        mock_alternate_service_type.assert_called_once_with(
            'image', allowed_types=('image',))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Flavor access interface."""

from novaclient import base
from novaclient.i18n import _


class FlavorAccess(base.Resource):
    def __repr__(self):
        return ("<FlavorAccess flavor id: %s, tenant id: %s>" %
                (self.flavor_id, self.tenant_id))


class FlavorAccessManager(base.ManagerWithFind):
    """Manage :class:`FlavorAccess` resources."""
    resource_class = FlavorAccess

    def list(self, **kwargs):
        # NOTE(mriedem): This looks a bit weird, you would normally expect this
        # method to just take a flavor arg, but it used to erroneously accept
        # flavor or tenant, but never actually implemented support for listing
        # flavor access by tenant. We leave the interface unchanged though for
        # backward compatibility.
        if kwargs.get('flavor'):
            return self._list('/flavors/%s/os-flavor-access' %
                              base.getid(kwargs['flavor']), 'flavor_access')
        raise NotImplementedError(_('Unknown list options.'))

    def add_tenant_access(self, flavor, tenant):
        """Add a tenant to the given flavor access list."""
        info = {'tenant': tenant}
        return self._action('addTenantAccess', flavor, info)

    def remove_tenant_access(self, flavor, tenant):
        """Remove a tenant from the given flavor access list."""
        info = {'tenant': tenant}
        return self._action('removeTenantAccess', flavor, info)

    def _action(self, action, flavor, info, **kwargs):
        """Perform a flavor action."""
        body = {action: info}
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/flavors/%s/action' % base.getid(flavor)
        resp, body = self.api.client.post(url, body=body)

        items = [self.resource_class(self, res)
                 for res in body['flavor_access']]

        return base.ListWithMeta(items, resp)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient.tests.unit.fixture_data import base


class V1(base.Fixture):

    base_url = 'os-quota-sets'

    def setUp(self):
        super(V1, self).setUp()

        uuid = '97f4c221-bff4-4578-b030-0df4ef119353'
        uuid2 = '97f4c221bff44578b0300df4ef119353'
        test_json = {'quota_set': self.test_quota('test')}
        self.headers = self.json_headers

        for u in ('test', 'tenant-id', 'tenant-id/defaults',
                  '%s/defaults' % uuid2, 'test/detail'):
            self.requests_mock.get(self.url(u),
                                   json=test_json,
                                   headers=self.headers)

        self.requests_mock.put(self.url(uuid),
                               json={'quota_set': self.test_quota(uuid)},
                               headers=self.headers)

        self.requests_mock.get(self.url(uuid),
                               json={'quota_set': self.test_quota(uuid)},
                               headers=self.headers)

        self.requests_mock.put(self.url(uuid2),
                               json={'quota_set': self.test_quota(uuid2)},
                               headers=self.headers)
        self.requests_mock.get(self.url(uuid2),
                               json={'quota_set': self.test_quota(uuid2)},
                               headers=self.headers)

        for u in ('test', uuid2):
            self.requests_mock.delete(self.url(u), status_code=202,
                                      headers=self.headers)

    def test_quota(self, tenant_id='test'):
        return {
            'id': tenant_id,
            'metadata_items': 1,
            'injected_file_content_bytes': 1,
            'injected_file_path_bytes': 1,
            'ram': 1,
            'fixed_ips': -1,
            'floating_ips': 1,
            'instances': 1,
            'injected_files': 1,
            'cores': 1,
            'key_pairs': 1,
            'security_groups': 1,
            'security_group_rules': 1,
            'server_groups': 1,
            'server_group_members': 1
        }


class V2_57(V1):
    """2.57 fixture data where there are no injected file or network resources
    """

    def test_quota(self, tenant_id='test'):
        return {
            'id': tenant_id,
            'metadata_items': 1,
            'ram': 1,
            'instances': 1,
            'cores': 1,
            'key_pairs': 1,
            'server_groups': 1,
            'server_group_members': 1
        }



# Copyright 2012 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
agent interface
"""

from novaclient import base

# NOTE(takashin): The os-agents APIs have been removed
# in https://review.opendev.org/c/openstack/nova/+/749309 .
# But the following API bindings remains as there are
# because the python-openstackclient depends on them.


class Agent(base.Resource):
    def __repr__(self):
        return "<Agent: %s>" % self.agent

    def _add_details(self, info):
        dico = 'resource' in info and info['resource'] or info
        for (k, v) in dico.items():
            setattr(self, k, v)


class AgentsManager(base.ManagerWithFind):
    resource_class = Agent

    def list(self, hypervisor=None):
        """List all agent builds."""
        url = "/os-agents"
        if hypervisor:
            url = "/os-agents?hypervisor=%s" % hypervisor
        return self._list(url, "agents")

    def _build_update_body(self, version, url, md5hash):
        return {'para': {'version': version,
                         'url': url,
                         'md5hash': md5hash}}

    def update(self, id, version,
               url, md5hash):
        """Update an existing agent build."""
        body = self._build_update_body(version, url, md5hash)
        return self._update('/os-agents/%s' % id, body, 'agent')

    def create(self, os, architecture, version,
               url, md5hash, hypervisor):
        """Create a new agent build."""
        body = {'agent': {'hypervisor': hypervisor,
                          'os': os,
                          'architecture': architecture,
                          'version': version,
                          'url': url,
                          'md5hash': md5hash}}
        return self._create('/os-agents', body, 'agent')

    def delete(self, id):
        """
        Deletes an existing agent build.

        :param id: The agent's id to delete
        :returns: An instance of novaclient.base.TupleWithMeta
        """
        return self._delete('/os-agents/%s' % id)



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from novaclient import client
from novaclient import exceptions
from novaclient.i18n import _
from novaclient.v2 import agents
from novaclient.v2 import aggregates
from novaclient.v2 import assisted_volume_snapshots
from novaclient.v2 import availability_zones
from novaclient.v2 import flavor_access
from novaclient.v2 import flavors
from novaclient.v2 import hypervisors
from novaclient.v2 import images
from novaclient.v2 import instance_action
from novaclient.v2 import instance_usage_audit_log
from novaclient.v2 import keypairs
from novaclient.v2 import limits
from novaclient.v2 import migrations
from novaclient.v2 import networks
from novaclient.v2 import quota_classes
from novaclient.v2 import quotas
from novaclient.v2 import server_external_events
from novaclient.v2 import server_groups
from novaclient.v2 import server_migrations
from novaclient.v2 import servers
from novaclient.v2 import services
from novaclient.v2 import usage
from novaclient.v2 import versions
from novaclient.v2 import volumes


class Client(object):
    """Top-level object to access the OpenStack Compute API.

    .. warning:: All scripts and projects should not initialize this class
      directly. It should be done via `novaclient.client.Client` interface.
    """

    def __init__(self,
                 api_version=None,
                 auth=None,
                 auth_token=None,
                 auth_url=None,
                 cacert=None,
                 cert=None,
                 direct_use=True,
                 endpoint_override=None,
                 endpoint_type='publicURL',
                 extensions=None,
                 http_log_debug=False,
                 insecure=False,
                 logger=None,
                 os_cache=False,
                 password=None,
                 project_domain_id=None,
                 project_domain_name=None,
                 project_id=None,
                 project_name=None,
                 region_name=None,
                 service_name=None,
                 service_type='compute',
                 session=None,
                 timeout=None,
                 timings=False,
                 user_domain_id=None,
                 user_domain_name=None,
                 user_id=None,
                 username=None,
                 **kwargs):
        """Initialization of Client object.

        :param api_version: Compute API version
        :type api_version: novaclient.api_versions.APIVersion
        :param str auth: Auth
        :param str auth_token: Auth token
        :param str auth_url: Auth URL
        :param str cacert: ca-certificate
        :param str cert: certificate
        :param bool direct_use: Inner variable of novaclient. Do not use it
            outside novaclient. It's restricted.
        :param str endpoint_override: Bypass URL
        :param str endpoint_type: Endpoint Type
        :param str extensions: Extensions
        :param bool http_log_debug: Enable debugging for HTTP connections
        :param bool insecure: Allow insecure
        :param logging.Logger logger: Logger instance to be used for all
            logging stuff
        :param str password: User password
        :param bool os_cache: OS cache
        :param str project_domain_id: ID of project domain
        :param str project_domain_name: Name of project domain
        :param str project_id: Project/Tenant ID
        :param str project_name: Project/Tenant name
        :param str region_name: Region Name
        :param str service_name: Service Name
        :param str service_type: Service Type
        :param str session: Session
        :param float timeout: API timeout, None or 0 disables
        :param bool timings: Timings
        :param str user_domain_id: ID of user domain
        :param str user_domain_name: Name of user domain
        :param str user_id: User ID
        :param str username: Username
        """
        if direct_use:
            raise exceptions.Forbidden(
                403, _("'novaclient.v2.client.Client' is not designed to be "
                       "initialized directly. It is inner class of "
                       "novaclient. You should use "
                       "'novaclient.client.Client' instead. Related lp "
                       "bug-report: 1493576"))

        # NOTE(cyeoh): In the novaclient context (unlike Nova) the
        # project_id is not the same as the tenant_id. Here project_id
        # is a name (what the Nova API often refers to as a project or
        # tenant name) and tenant_id is a UUID (what the Nova API
        # often refers to as a project_id or tenant_id).

        self.project_id = project_id
        self.project_name = project_name
        self.user_id = user_id
        self.flavors = flavors.FlavorManager(self)
        self.flavor_access = flavor_access.FlavorAccessManager(self)
        self.glance = images.GlanceManager(self)
        self.limits = limits.LimitsManager(self)
        self.servers = servers.ServerManager(self)
        self.versions = versions.VersionManager(self)

        # extensions
        self.agents = agents.AgentsManager(self)
        self.volumes = volumes.VolumeManager(self)
        self.keypairs = keypairs.KeypairManager(self)
        self.neutron = networks.NeutronManager(self)
        self.quota_classes = quota_classes.QuotaClassSetManager(self)
        self.quotas = quotas.QuotaSetManager(self)
        self.usage = usage.UsageManager(self)
        self.aggregates = aggregates.AggregateManager(self)
        self.hypervisors = hypervisors.HypervisorManager(self)
        self.hypervisor_stats = hypervisors.HypervisorStatsManager(self)
        self.services = services.ServiceManager(self)
        self.os_cache = os_cache
        self.availability_zones = \
            availability_zones.AvailabilityZoneManager(self)
        self.server_groups = server_groups.ServerGroupsManager(self)
        self.server_migrations = \
            server_migrations.ServerMigrationsManager(self)

        # V2.0 extensions:
        # NOTE(andreykurilin): tenant_networks extension is
        #   deprecated now, which is why it is not initialized by default.
        self.assisted_volume_snapshots = \
            assisted_volume_snapshots.AssistedSnapshotManager(self)
        self.instance_action = instance_action.InstanceActionManager(self)
        self.instance_usage_audit_log = \
            instance_usage_audit_log.InstanceUsageAuditLogManager(self)
        self.migrations = migrations.MigrationManager(self)
        self.server_external_events = \
            server_external_events.ServerExternalEventManager(self)

        self.logger = logger or logging.getLogger(__name__)

        # Add in any extensions...
        if extensions:
            for extension in extensions:
                if extension.manager_class:
                    setattr(self, extension.name,
                            extension.manager_class(self))

        self.client = client._construct_http_client(
            api_version=api_version,
            auth=auth,
            auth_token=auth_token,
            auth_url=auth_url,
            cacert=cacert,
            cert=cert,
            endpoint_override=endpoint_override,
            endpoint_type=endpoint_type,
            http_log_debug=http_log_debug,
            insecure=insecure,
            logger=self.logger,
            os_cache=self.os_cache,
            password=password,
            project_domain_id=project_domain_id,
            project_domain_name=project_domain_name,
            project_id=project_id,
            project_name=project_name,
            region_name=region_name,
            service_name=service_name,
            service_type=service_type,
            session=session,
            timeout=timeout,
            timings=timings,
            user_domain_id=user_domain_id,
            user_domain_name=user_domain_name,
            user_id=user_id,
            username=username,
            **kwargs)

    @property
    def api_version(self):
        return self.client.api_version

    @api_version.setter
    def api_version(self, value):
        self.client.api_version = value

    def __enter__(self):
        raise exceptions.InvalidUsage(_(
            "NovaClient instance can't be used as a context manager "
            "since it is redundant in case of SessionClient."))

    def __exit__(self, t, v, tb):
        # do not do anything
        pass

    def get_timings(self):
        return self.client.get_timings()

    def reset_timings(self):
        self.client.reset_timings()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
from urllib import parse

import tempest.lib.cli.base

from novaclient import client
from novaclient.tests.functional import base


class TestAuthentication(base.ClientTestBase):

    def _get_url(self, identity_api_version):
        url = parse.urlparse(self.cli_clients.uri)
        return parse.urlunparse((url.scheme, url.netloc,
                                 '/identity/v%s' % identity_api_version,
                                 url.params, url.query,
                                 url.fragment))

    def nova_auth_with_password(self, action, identity_api_version):
        flags = (
            f'--os-username {self.cli_clients.username} '
            f'--os-tenant-name {self.cli_clients.tenant_name} '
            f'--os-password {self.cli_clients.password} '
            f'--os-auth-url {self._get_url(identity_api_version)} '
            f'--os-endpoint-type publicURL'
        )
        if self.cacert:
            flags = f'{flags} --os-cacert {self.cacert}'
        if self.cert:
            flags = f'{flags} --os-cert {self.cert}'
        if self.cli_clients.insecure:
            flags = f'{flags} --insecure'

        return tempest.lib.cli.base.execute(
            "nova", action, flags, cli_dir=self.cli_clients.cli_dir)

    def nova_auth_with_token(self, identity_api_version):
        auth_ref = self.client.client.session.auth.get_access(
            self.client.client.session)
        token = auth_ref.auth_token
        auth_url = self._get_url(identity_api_version)
        kw = {}
        if identity_api_version == "3":
            kw["project_domain_id"] = self.project_domain_id
        nova = client.Client("2", auth_token=token, auth_url=auth_url,
                             project_name=self.project_name,
                             cacert=self.cacert, cert=self.cert,
                             **kw)
        nova.servers.list()

        # NOTE(andreykurilin): tempest.lib.cli.base.execute doesn't allow to
        #   pass 'env' argument to subprocess.Popen for overriding the current
        #   process' environment.
        #   When one of OS_AUTH_TYPE or OS_AUTH_PLUGIN environment variables
        #   presents, keystoneauth1 can load the wrong auth plugin with wrong
        #   expected cli arguments. To avoid this case, we need to modify
        #   current environment.
        # TODO(andreykurilin): tempest.lib.cli.base.execute is quite simple
        #   method that can be replaced by subprocess.check_output direct call
        #   with passing env argument to avoid modifying the current process
        #   environment. or we probably can propose a change to tempest.
        os.environ.pop("OS_AUTH_TYPE", None)
        os.environ.pop("OS_AUTH_PLUGIN", None)

        flags = (
            f'--os-tenant-name {self.project_name} '
            f'--os-token {token} '
            f'--os-auth-url {auth_url} '
            f'--os-endpoint-type publicURL'
        )
        if self.cacert:
            flags = f'{flags} --os-cacert {self.cacert}'
        if self.cert:
            flags = f'{flags} --os-cert {self.cert}'
        if self.cli_clients.insecure:
            flags = f'{flags} --insecure'

        tempest.lib.cli.base.execute(
            "nova", "list", flags, cli_dir=self.cli_clients.cli_dir)

    def test_auth_via_keystone_v3(self):
        self.nova_auth_with_password("list", identity_api_version="3")
        self.nova_auth_with_token(identity_api_version="3")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient.tests.unit.fixture_data import base


class Fixture(base.Fixture):

    base_url = 'os-agents'

    def setUp(self):
        super(Fixture, self).setUp()

        post_os_agents = {
            'agent': {
                'url': '/xxx/xxx/xxx',
                'hypervisor': 'kvm',
                'md5hash': 'add6bb58e139be103324d04d82d8f546',
                'version': '7.0',
                'architecture': 'x86',
                'os': 'win',
                'id': 1
            }
        }

        self.requests_mock.post(self.url(),
                                json=post_os_agents,
                                headers=self.json_headers)

        put_os_agents_1 = {
            "agent": {
                "url": "/yyy/yyyy/yyyy",
                "version": "8.0",
                "md5hash": "add6bb58e139be103324d04d82d8f546",
                'id': 1
            }
        }

        self.requests_mock.put(self.url(1),
                               json=put_os_agents_1,
                               headers=self.json_headers)

        self.requests_mock.delete(self.url(1),
                                  headers=self.json_headers,
                                  status_code=202)



# Copyright (C) 2013, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Assisted volume snapshots - to be used by Cinder and not end users.
"""

from oslo_serialization import jsonutils

from novaclient import base


class Snapshot(base.Resource):
    def __repr__(self):
        return "<Snapshot: %s>" % self.id

    def delete(self):
        """
        Delete this snapshot.

        :returns: An instance of novaclient.base.TupleWithMeta
        """
        return self.manager.delete(self)


class AssistedSnapshotManager(base.Manager):
    resource_class = Snapshot

    def create(self, volume_id, create_info):
        body = {'snapshot': {'volume_id': volume_id,
                             'create_info': create_info}}
        return self._create('/os-assisted-volume-snapshots', body, 'snapshot')

    def delete(self, snapshot, delete_info):
        """
        Delete a specified assisted volume snapshot.

        :param snapshot: an assisted volume snapshot to delete
        :param delete_info: Information for snapshot deletion
        :returns: An instance of novaclient.base.TupleWithMeta
        """
        return self._delete("/os-assisted-volume-snapshots/%s?delete_info=%s" %
                            (base.getid(snapshot),
                             jsonutils.dumps(delete_info)))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base
from novaclient import utils


class TestHypervisors(base.ClientTestBase):

    COMPUTE_API_VERSION = "2.1"

    def _test_list(self, cpu_info_type, uuid_as_id=False):
        hypervisors = self.client.hypervisors.list()
        if not len(hypervisors):
            self.fail("No hypervisors detected.")
        for hypervisor in hypervisors:
            self.assertIsInstance(hypervisor.cpu_info, cpu_info_type)
            if uuid_as_id:
                # microversion >= 2.53 returns a uuid for the id
                self.assertFalse(utils.is_integer_like(hypervisor.id),
                                 'Expected hypervisor.id to be a UUID.')
                self.assertFalse(
                    utils.is_integer_like(hypervisor.service['id']),
                    'Expected hypervisor.service.id to be a UUID.')
            else:
                self.assertTrue(utils.is_integer_like(hypervisor.id),
                                'Expected hypervisor.id to be an integer.')
                self.assertTrue(
                    utils.is_integer_like(hypervisor.service['id']),
                    'Expected hypervisor.service.id to be an integer.')

    def test_list(self):
        self._test_list(str)



# Copyright 2017 Huawei Technologies Co.,LTD.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions

from novaclient.tests.functional import base


class TestQuotaClassesNovaClient(base.ClientTestBase):
    """Nova quota classes functional tests for the v2.1 microversion."""

    COMPUTE_API_VERSION = '2.1'

    # The list of quota class resources we expect in the output table.
    _included_resources = ['instances', 'cores', 'ram',
                           'floating_ips', 'fixed_ips', 'metadata_items',
                           'injected_files', 'injected_file_content_bytes',
                           'injected_file_path_bytes', 'key_pairs',
                           'security_groups', 'security_group_rules']

    # The list of quota class resources we do not expect in the output table.
    _excluded_resources = ['server_groups', 'server_group_members']

    # Any resources that are not shown but can be updated. For example, before
    # microversion 2.50 you can update server_groups and server_groups_members
    # quota class values but they are not shown in the GET response.
    _extra_update_resources = _excluded_resources

    # The list of resources which are blocked from being updated.
    _blocked_update_resources = []

    def _get_quota_class_name(self):
        """Returns a fake quota class name specific to this test class."""
        return 'fake-class-%s' % self.COMPUTE_API_VERSION.replace('.', '-')

    def _verify_quota_class_show_output(self, output, expected_values):
        # Assert that the expected key/value pairs are in the output table
        for quota_name in self._included_resources:
            # First make sure the resource is actually in expected quota.
            self.assertIn(quota_name, expected_values)
            expected_value = expected_values[quota_name]
            actual_value = self._get_value_from_the_table(output, quota_name)
            self.assertEqual(expected_value, actual_value)

        # Now make sure anything that we don't expect in the output table is
        # actually not showing up.
        for quota_name in self._excluded_resources:
            # ValueError is raised when the key isn't found in the table.
            self.assertRaises(ValueError,
                              self._get_value_from_the_table,
                              output, quota_name)

    def test_quota_class_show(self):
        """Tests showing quota class values for a fake non-existing quota
        class. The API will return the defaults if the quota class does not
        actually exist. We use a fake class to avoid any interaction with the
        real default quota class values.
        """
        default_quota_class_set = self.client.quota_classes.get('default')
        default_values = {
            quota_name: str(getattr(default_quota_class_set, quota_name))
            for quota_name in self._included_resources
        }
        output = self.nova('quota-class-show %s' %
                           self._get_quota_class_name())
        self._verify_quota_class_show_output(output, default_values)

    def test_quota_class_update(self):
        """Tests updating a fake quota class. The way this works in the API
        is that if the quota class is not found, it is created. So in this
        test we can use a fake quota class with fake values and they will all
        get set. We don't use the default quota class because it is global
        and we don't want to interfere with other tests.
        """
        class_name = self._get_quota_class_name()
        params = [class_name]
        expected_values = {}
        for quota_name in (
                self._included_resources + self._extra_update_resources):
            params.append("--%s 99" % quota_name.replace("_", "-"))
            expected_values[quota_name] = '99'

        # Note that the quota-class-update CLI doesn't actually output any
        # information from the response.
        self.nova("quota-class-update", params=" ".join(params))
        # Assert the results using the quota-class-show output.
        output = self.nova('quota-class-show %s' % class_name)
        self._verify_quota_class_show_output(output, expected_values)

        # Assert that attempting to update resources that are blocked will
        # result in a failure.
        for quota_name in self._blocked_update_resources:
            self.assertRaises(
                exceptions.CommandFailed,
                self.nova, "quota-class-update %s --%s 99" %
                           (class_name, quota_name.replace("_", "-")))


class TestQuotasNovaClient2_50(TestQuotaClassesNovaClient):
    """Nova quota classes functional tests for the v2.50 microversion."""

    COMPUTE_API_VERSION = '2.50'

    # The 2.50 microversion added the server_groups and server_group_members
    # to the response, and filtered out floating_ips, fixed_ips,
    # security_groups and security_group_members, similar to the 2.36
    # microversion in the os-quota-sets API.
    _included_resources = ['instances', 'cores', 'ram', 'metadata_items',
                           'injected_files', 'injected_file_content_bytes',
                           'injected_file_path_bytes', 'key_pairs',
                           'server_groups', 'server_group_members']

    # The list of quota class resources we do not expect in the output table.
    _excluded_resources = ['floating_ips', 'fixed_ips',
                           'security_groups', 'security_group_rules']

    # In 2.50, server_groups and server_group_members can be both updated
    # in a PUT request and shown in a GET response.
    _extra_update_resources = []

    # In 2.50, you can't update the network-related resources.
    _blocked_update_resources = _excluded_resources



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64
import io
import os
import tempfile
from unittest import mock

from novaclient import api_versions
from novaclient import exceptions
from novaclient.tests.unit.fixture_data import client
from novaclient.tests.unit.fixture_data import floatingips
from novaclient.tests.unit.fixture_data import servers as data
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import servers


class ServersTest(utils.FixturedTestCase):

    client_fixture_class = client.V1
    data_fixture_class = data.V1
    api_version = None
    supports_files = True

    def setUp(self):
        super(ServersTest, self).setUp()
        self.useFixture(floatingips.FloatingFixture(self.requests_mock))
        if self.api_version:
            self.cs.api_version = api_versions.APIVersion(self.api_version)

    def _get_server_create_default_nics(self):
        """Callback for default nics kwarg when creating a server.
        """
        return None

    def test_list_servers(self):
        sl = self.cs.servers.list()
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/detail')
        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_filter_servers_unicode(self):
        sl = self.cs.servers.list(search_opts={'name': 't€sting'})
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/detail?name=t%E2%82%ACsting')
        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_list_all_servers(self):
        # use marker just to identify this call in fixtures
        sl = self.cs.servers.list(limit=-1, marker=1234)
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)

        self.assertEqual(2, len(sl))

        self.assertEqual(self.requests_mock.request_history[-2].method, 'GET')
        self.assertEqual(self.requests_mock.request_history[-2].path_url,
                         '/servers/detail?marker=1234')
        self.assert_called('GET', '/servers/detail?marker=5678')

        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_filter_servers_unlocked(self):
        # calling the cs.servers.list python binding
        # will fail before 2.73 microversion.
        e = self.assertRaises(exceptions.UnsupportedAttribute,
                              self.cs.servers.list,
                              search_opts={'locked': False})
        self.assertIn("'locked' argument is only allowed since "
                      "microversion 2.73.", str(e))

    def test_filter_without_config_drive(self):
        sl = self.cs.servers.list(search_opts={'config_drive': None})
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/detail')
        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_list_servers_undetailed(self):
        sl = self.cs.servers.list(detailed=False)
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers')
        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_list_servers_with_marker_limit(self):
        sl = self.cs.servers.list(marker=1234, limit=2)
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/detail?limit=2&marker=1234')
        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_list_servers_sort_single(self):
        sl = self.cs.servers.list(sort_keys=['display_name'],
                                  sort_dirs=['asc'])
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'GET',
            '/servers/detail?sort_dir=asc&sort_key=display_name')
        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_list_servers_sort_multiple(self):
        sl = self.cs.servers.list(sort_keys=['display_name', 'id'],
                                  sort_dirs=['asc', 'desc'])
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'GET',
            ('/servers/detail?sort_dir=asc&sort_dir=desc&'
             'sort_key=display_name&sort_key=id'))
        for s in sl:
            self.assertIsInstance(s, servers.Server)

    def test_get_server_details(self):
        s = self.cs.servers.get(1234)
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/1234')
        self.assertIsInstance(s, servers.Server)
        self.assertEqual(1234, s.id)
        self.assertEqual('BUILD', s.status)

    def test_get_server_promote_details(self):
        s1 = self.cs.servers.list(detailed=False)[0]
        s2 = self.cs.servers.list(detailed=True)[0]
        self.assertNotEqual(s1._info, s2._info)
        s1.get()
        self.assertEqual(s1._info, s2._info)

    def test_create_server(self):
        kwargs = {}
        if self.supports_files:
            kwargs['files'] = {
                '/etc/passwd': 'some data',             # a file
                '/tmp/foo.txt': io.StringIO('data'),    # a stream
            }
        s = self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            meta={'foo': 'bar'},
            userdata="hello moto",
            key_name="fakekey",
            nics=self._get_server_create_default_nics(),
            **kwargs
        )
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')
        self.assertIsInstance(s, servers.Server)

    def test_create_server_boot_from_volume_with_nics(self):
        old_boot = self.cs.servers._boot

        nics = [{'net-id': '11111111-1111-1111-1111-111111111111',
                 'v4-fixed-ip': '10.0.0.7'}]
        bdm = {"volume_size": "1",
               "volume_id": "11111111-1111-1111-1111-111111111111",
               "delete_on_termination": "0",
               "device_name": "vda"}

        def wrapped_boot(url, key, *boot_args, **boot_kwargs):
            self.assertEqual(boot_kwargs['block_device_mapping'], bdm)
            self.assertEqual(boot_kwargs['nics'], nics)
            return old_boot(url, key, *boot_args, **boot_kwargs)

        @mock.patch.object(self.cs.servers, '_boot', wrapped_boot)
        def test_create_server_from_volume():
            s = self.cs.servers.create(
                name="My server",
                image=1,
                flavor=1,
                meta={'foo': 'bar'},
                userdata="hello moto",
                key_name="fakekey",
                block_device_mapping=bdm,
                nics=nics
            )
            self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
            self.assert_called('POST', '/servers')
            self.assertIsInstance(s, servers.Server)

        test_create_server_from_volume()

    def test_create_server_boot_from_volume_bdm_v2(self):
        old_boot = self.cs.servers._boot

        bdm = [{"volume_size": "1",
                "volume_id": "11111111-1111-1111-1111-111111111111",
                "delete_on_termination": "0",
                "device_name": "vda"}]

        def wrapped_boot(url, key, *boot_args, **boot_kwargs):
            self.assertEqual(boot_kwargs['block_device_mapping_v2'], bdm)
            return old_boot(url, key, *boot_args, **boot_kwargs)

        with mock.patch.object(self.cs.servers, '_boot', wrapped_boot):
            s = self.cs.servers.create(
                name="My server",
                image=1,
                flavor=1,
                meta={'foo': 'bar'},
                userdata="hello moto",
                key_name="fakekey",
                block_device_mapping_v2=bdm,
                nics=self._get_server_create_default_nics()
            )
            self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
            self.assert_called('POST', '/servers')
            self.assertIsInstance(s, servers.Server)

    def test_create_server_boot_with_nics_ipv6(self):
        old_boot = self.cs.servers._boot
        nics = [{'net-id': '11111111-1111-1111-1111-111111111111',
                 'v6-fixed-ip': '2001:db9:0:1::10'}]

        def wrapped_boot(url, key, *boot_args, **boot_kwargs):
            self.assertEqual(boot_kwargs['nics'], nics)
            return old_boot(url, key, *boot_args, **boot_kwargs)

        with mock.patch.object(self.cs.servers, '_boot', wrapped_boot):
            s = self.cs.servers.create(
                name="My server",
                image=1,
                flavor=1,
                meta={'foo': 'bar'},
                userdata="hello moto",
                key_name="fakekey",
                nics=nics
            )
            self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
            self.assert_called('POST', '/servers')
            self.assertIsInstance(s, servers.Server)

    def test_create_server_boot_with_address(self):
        old_boot = self.cs.servers._boot
        access_ip_v6 = '::1'
        access_ip_v4 = '10.10.10.10'

        def wrapped_boot(url, key, *boot_args, **boot_kwargs):
            self.assertEqual(boot_kwargs['access_ip_v6'], access_ip_v6)
            self.assertEqual(boot_kwargs['access_ip_v4'], access_ip_v4)
            return old_boot(url, key, *boot_args, **boot_kwargs)

        with mock.patch.object(self.cs.servers, '_boot', wrapped_boot):
            s = self.cs.servers.create(
                name="My server",
                image=1,
                flavor=1,
                meta={'foo': 'bar'},
                userdata="hello moto",
                key_name="fakekey",
                access_ip_v6=access_ip_v6,
                access_ip_v4=access_ip_v4,
                nics=self._get_server_create_default_nics()
            )
            self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
            self.assert_called('POST', '/servers')
            self.assertIsInstance(s, servers.Server)

    def test_create_server_userdata_file_object(self):
        kwargs = {}
        if self.supports_files:
            kwargs['files'] = {
                '/etc/passwd': 'some data',             # a file
                '/tmp/foo.txt': io.StringIO('data'),   # a stream
            }
        s = self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            meta={'foo': 'bar'},
            userdata=io.StringIO('hello moto'),
            nics=self._get_server_create_default_nics(),
            **kwargs
        )
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')
        self.assertIsInstance(s, servers.Server)

    def test_create_server_userdata_unicode(self):
        kwargs = {}
        if self.supports_files:
            kwargs['files'] = {
                '/etc/passwd': 'some data',             # a file
                '/tmp/foo.txt': io.StringIO('data'),   # a stream
            }
        s = self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            meta={'foo': 'bar'},
            userdata='こんにちは',
            key_name="fakekey",
            nics=self._get_server_create_default_nics(),
            **kwargs
        )
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')
        self.assertIsInstance(s, servers.Server)

    def test_create_server_userdata_utf8(self):
        kwargs = {}
        if self.supports_files:
            kwargs['files'] = {
                '/etc/passwd': 'some data',             # a file
                '/tmp/foo.txt': io.StringIO('data'),   # a stream
            }
        s = self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            meta={'foo': 'bar'},
            userdata='こんにちは',
            key_name="fakekey",
            nics=self._get_server_create_default_nics(),
            **kwargs
        )
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')
        self.assertIsInstance(s, servers.Server)

    def test_create_server_admin_pass(self):
        test_password = "test-pass"
        test_key = "fakekey"
        s = self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            admin_pass=test_password,
            key_name=test_key,
            nics=self._get_server_create_default_nics()
        )
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')
        self.assertIsInstance(s, servers.Server)
        body = self.requests_mock.last_request.json()
        self.assertEqual(test_password, body['server']['adminPass'])

    def test_create_server_userdata_bin(self):
        kwargs = {}
        if self.supports_files:
            kwargs['files'] = {
                '/etc/passwd': 'some data',             # a file
                '/tmp/foo.txt': io.StringIO('data'),   # a stream
            }
        with tempfile.TemporaryFile(mode='wb+') as bin_file:
            original_data = os.urandom(1024)
            bin_file.write(original_data)
            bin_file.flush()
            bin_file.seek(0)
            s = self.cs.servers.create(
                name="My server",
                image=1,
                flavor=1,
                meta={'foo': 'bar'},
                userdata=bin_file,
                key_name="fakekey",
                nics=self._get_server_create_default_nics(),
                **kwargs
            )
            self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
            self.assert_called('POST', '/servers')
            self.assertIsInstance(s, servers.Server)
            # verify userdata matches original
            body = self.requests_mock.last_request.json()
            transferred_data = body['server']['user_data']
            transferred_data = base64.b64decode(transferred_data)
            self.assertEqual(original_data, transferred_data)

    def _create_disk_config(self, disk_config):
        s = self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            disk_config=disk_config,
            nics=self._get_server_create_default_nics(),
        )
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')
        self.assertIsInstance(s, servers.Server)

        # verify disk config param was used in the request:
        server = self.requests_mock.last_request.json()['server']
        self.assertIn('OS-DCF:diskConfig', server)
        self.assertEqual(disk_config, server['OS-DCF:diskConfig'])

    def test_create_server_disk_config_auto(self):
        self._create_disk_config('AUTO')

    def test_create_server_disk_config_manual(self):
        self._create_disk_config('MANUAL')

    def test_create_server_return_reservation_id(self):
        s = self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            reservation_id=True,
            nics=self._get_server_create_default_nics()
        )
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')

    def test_update_server(self):
        s = self.cs.servers.get(1234)

        # Update via instance
        s.update(name='hi')
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('PUT', '/servers/1234')
        s.update(name='hi')
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('PUT', '/servers/1234')

        # Silly, but not an error
        s.update()

        # Update via manager
        self.cs.servers.update(s, name='hi')
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('PUT', '/servers/1234')

    def test_delete_server(self):
        s = self.cs.servers.get(1234)
        ret = s.delete()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234')
        ret = self.cs.servers.delete(1234)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234')
        ret = self.cs.servers.delete(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234')

    def test_delete_server_meta(self):
        ret = self.cs.servers.delete_meta(1234, ['test_key'])
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234/metadata/test_key')

    def test_set_server_meta(self):
        m = self.cs.servers.set_meta(1234, {'test_key': 'test_value'})
        self.assert_request_id(m, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/metadata',
                           {'metadata': {'test_key': 'test_value'}})

    def test_set_server_meta_item(self):
        m = self.cs.servers.set_meta_item(1234, 'test_key', 'test_value')
        self.assert_request_id(m, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('PUT', '/servers/1234/metadata/test_key',
                           {'meta': {'test_key': 'test_value'}})

    def test_get_server_meta(self):
        m = self.cs.servers.get_meta(1234, 'Server Label')
        self.assert_request_id(m, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/1234/metadata/Server%20Label')
        self.assertEqual(m, {'meta': {
            'Server Label': 'Web Head 1'
        }})

    def test_list_server_meta(self):
        m = self.cs.servers.list_meta(1234)
        self.assert_request_id(m, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/1234/metadata')
        self.assertEqual(m, {'metadata': {
            'Server Label': 'Web Head 1',
            'Image Version': '2.1'
        }})

    def test_find(self):
        server = self.cs.servers.find(name='sample-server')
        self.assert_request_id(server, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/1234')
        self.assertEqual('sample-server', server.name)
        # The networks should be sorted.
        networks = server.networks
        self.assertEqual(2, len(networks))
        labels = list(networks)  # returns the dict keys
        self.assertEqual('private', labels[0])
        self.assertEqual('public', labels[1])

        self.assertRaises(exceptions.NoUniqueMatch, self.cs.servers.find,
                          flavor={"id": 1, "name": "256 MiB Server"})

        sl = self.cs.servers.findall(flavor={"id": 1,
                                             "name": "256 MiB Server"})
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual([1234, 5678, 9012], [s.id for s in sl])

    def test_reboot_server(self):
        s = self.cs.servers.get(1234)
        ret = s.reboot()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.reboot(s, reboot_type='HARD')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_rebuild_server(self):
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image=1)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.rebuild(s, image=1)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = s.rebuild(image=1, password='5678')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.rebuild(s, image=1, password='5678')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def _rebuild_resize_disk_config(self, disk_config, operation="rebuild"):
        s = self.cs.servers.get(1234)

        if operation == "rebuild":
            ret = s.rebuild(image=1, disk_config=disk_config)
        elif operation == "resize":
            ret = s.resize(flavor=1, disk_config=disk_config)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

        # verify disk config param was used in the request:
        d = self.requests_mock.last_request.json()[operation]
        self.assertIn('OS-DCF:diskConfig', d)
        self.assertEqual(disk_config, d['OS-DCF:diskConfig'])

    def test_rebuild_server_disk_config_auto(self):
        self._rebuild_resize_disk_config('AUTO')

    def test_rebuild_server_disk_config_manual(self):
        self._rebuild_resize_disk_config('MANUAL')

    def test_rebuild_server_preserve_ephemeral(self):
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image=1, preserve_ephemeral=True)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        d = self.requests_mock.last_request.json()['rebuild']
        self.assertIn('preserve_ephemeral', d)
        self.assertTrue(d['preserve_ephemeral'])

    def test_rebuild_server_name_meta_files(self):
        files = {'/etc/passwd': 'some data'}
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image=1, name='new', meta={'foo': 'bar'}, files=files)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        d = self.requests_mock.last_request.json()['rebuild']
        self.assertEqual('new', d['name'])
        self.assertEqual({'foo': 'bar'}, d['metadata'])
        self.assertEqual('/etc/passwd',
                         d['personality'][0]['path'])

    def test_resize_server(self):
        s = self.cs.servers.get(1234)
        ret = s.resize(flavor=1)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        self.cs.servers.resize(s, flavor=1)
        self.assert_called('POST', '/servers/1234/action')

    def test_resize_server_disk_config_auto(self):
        self._rebuild_resize_disk_config('AUTO', 'resize')

    def test_resize_server_disk_config_manual(self):
        self._rebuild_resize_disk_config('MANUAL', 'resize')

    def test_confirm_resized_server(self):
        s = self.cs.servers.get(1234)
        ret = s.confirm_resize()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        self.cs.servers.confirm_resize(s)
        self.assert_called('POST', '/servers/1234/action')

    def test_revert_resized_server(self):
        s = self.cs.servers.get(1234)
        ret = s.revert_resize()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.revert_resize(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_migrate_server(self):
        s = self.cs.servers.get(1234)
        ret = s.migrate()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.migrate(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_stop(self):
        s = self.cs.servers.get(1234)
        ret = s.stop()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.stop(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_force_delete(self):
        s = self.cs.servers.get(1234)
        ret = s.force_delete()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.force_delete(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_restore(self):
        s = self.cs.servers.get(1234)
        ret = s.restore()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.restore(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_start(self):
        s = self.cs.servers.get(1234)
        ret = s.start()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.start(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_rescue(self):
        s = self.cs.servers.get(1234)
        ret = s.rescue()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.rescue(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_rescue_password(self):
        s = self.cs.servers.get(1234)
        ret = s.rescue(password='asdf')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rescue': {'adminPass': 'asdf'}})
        ret = self.cs.servers.rescue(s, password='asdf')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rescue': {'adminPass': 'asdf'}})

    def test_rescue_image(self):
        s = self.cs.servers.get(1234)
        ret = s.rescue(image=1)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rescue': {'rescue_image_ref': 1}})
        ret = self.cs.servers.rescue(s, image=1)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rescue': {'rescue_image_ref': 1}})

    def test_unrescue(self):
        s = self.cs.servers.get(1234)
        ret = s.unrescue()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.unrescue(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_lock(self):
        s = self.cs.servers.get(1234)
        ret = s.lock()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.lock(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_unlock(self):
        s = self.cs.servers.get(1234)
        ret = s.unlock()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.unlock(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_backup(self):
        s = self.cs.servers.get(1234)
        sb = s.backup('back1', 'daily', 1)
        self.assert_request_id(sb, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        sb = self.cs.servers.backup(s, 'back1', 'daily', 2)
        self.assert_request_id(sb, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_get_console_output_without_length(self):
        success = 'foo'
        s = self.cs.servers.get(1234)
        co = s.get_console_output()
        self.assert_request_id(co, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual(success, s.get_console_output())
        self.assert_called('POST', '/servers/1234/action')

        co = self.cs.servers.get_console_output(s)
        self.assert_request_id(co, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual(success, self.cs.servers.get_console_output(s))
        self.assert_called('POST', '/servers/1234/action')

    def test_get_console_output_with_length(self):
        success = 'foo'

        s = self.cs.servers.get(1234)
        co = s.get_console_output(length=50)
        self.assert_request_id(co, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual(success, s.get_console_output(length=50))
        self.assert_called('POST', '/servers/1234/action')

        co = self.cs.servers.get_console_output(s, length=50)
        self.assert_request_id(co, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual(success,
                         self.cs.servers.get_console_output(s, length=50))
        self.assert_called('POST', '/servers/1234/action')

    # Testing password methods with the following password and key
    #
    # Clear password: FooBar123
    #
    # RSA Private Key: novaclient/tests/unit/idfake.pem
    #
    # Encrypted password
    # OIuEuQttO8Rk93BcKlwHQsziDAnkAm/V6V8VPToA8ZeUaUBWwS0gwo2K6Y61Z96r
    # qG447iRz0uTEEYq3RAYJk1mh3mMIRVl27t8MtIecR5ggVVbz1S9AwXJQypDKl0ho
    # QFvhCBcMWPohyGewDJOhDbtuN1IoFI9G55ZvFwCm5y7m7B2aVcoLeIsJZE4PLsIw
    # /y5a6Z3/AoJZYGG7IH5WN88UROU3B9JZGFB2qtPLQTOvDMZLUhoPRIJeHiVSlo1N
    # tI2/++UsXVg3ow6ItqCJGgdNuGG5JB+bslDHWPxROpesEIHdczk46HCpHQN8f1sk
    # Hi/fmZZNQQqj1Ijq0caOIw==

    def test_get_password(self):
        s = self.cs.servers.get(1234)
        password = s.get_password('novaclient/tests/unit/idfake.pem')
        self.assert_request_id(password, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual('FooBar123', password)
        self.assert_called('GET', '/servers/1234/os-server-password')

    def test_get_password_without_key(self):
        s = self.cs.servers.get(1234)
        password = s.get_password()
        self.assert_request_id(password, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual(
            'OIuEuQttO8Rk93BcKlwHQsziDAnkAm/V6V8VPToA8ZeUaUBWwS0gwo2K6Y61Z96r'
            'qG447iRz0uTEEYq3RAYJk1mh3mMIRVl27t8MtIecR5ggVVbz1S9AwXJQypDKl0ho'
            'QFvhCBcMWPohyGewDJOhDbtuN1IoFI9G55ZvFwCm5y7m7B2aVcoLeIsJZE4PLsIw'
            '/y5a6Z3/AoJZYGG7IH5WN88UROU3B9JZGFB2qtPLQTOvDMZLUhoPRIJeHiVSlo1N'
            'tI2/++UsXVg3ow6ItqCJGgdNuGG5JB+bslDHWPxROpesEIHdczk46HCpHQN8f1sk'
            'Hi/fmZZNQQqj1Ijq0caOIw==', password)
        self.assert_called('GET', '/servers/1234/os-server-password')

    def test_clear_password(self):
        s = self.cs.servers.get(1234)
        ret = s.clear_password()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234/os-server-password')

    def test_get_server_diagnostics(self):
        s = self.cs.servers.get(1234)
        diagnostics = s.diagnostics()
        self.assert_request_id(diagnostics, fakes.FAKE_REQUEST_ID_LIST)
        self.assertIsNotNone(diagnostics)
        self.assert_called('GET', '/servers/1234/diagnostics')

        diagnostics_from_manager = self.cs.servers.diagnostics(1234)
        self.assert_request_id(diagnostics_from_manager,
                               fakes.FAKE_REQUEST_ID_LIST)
        self.assertIsNotNone(diagnostics_from_manager)
        self.assert_called('GET', '/servers/1234/diagnostics')

        self.assertEqual(diagnostics[1], diagnostics_from_manager[1])

    def test_get_vnc_console(self):
        s = self.cs.servers.get(1234)
        vc = s.get_vnc_console('novnc')
        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

        vc = self.cs.servers.get_vnc_console(s, 'novnc')
        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_get_spice_console(self):
        s = self.cs.servers.get(1234)
        sc = s.get_spice_console('spice-html5')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

        sc = self.cs.servers.get_spice_console(s, 'spice-html5')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_get_serial_console(self):
        s = self.cs.servers.get(1234)
        sc = s.get_serial_console('serial')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

        sc = self.cs.servers.get_serial_console(s, 'serial')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_get_rdp_console(self):
        s = self.cs.servers.get(1234)
        rc = s.get_rdp_console('rdp-html5')
        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

        rc = self.cs.servers.get_rdp_console(s, 'rdp-html5')
        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_get_console_url(self):
        s = self.cs.servers.get(1234)
        rc = s.get_console_url('novnc')
        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

        rc = self.cs.servers.get_console_url(s, 'novnc')
        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

        # test the case with invalid console type
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_console_url,
                          'invalid')

    def test_create_image(self):
        s = self.cs.servers.get(1234)
        im = s.create_image('123')
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        im = s.create_image('123', {})
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        im = self.cs.servers.create_image(s, '123')
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        im = self.cs.servers.create_image(s, '123', {})
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_live_migrate_server(self):
        s = self.cs.servers.get(1234)
        ret = s.live_migrate(host='hostname', block_migration=False,
                             disk_over_commit=False)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': False,
                                               'disk_over_commit': False}})
        ret = self.cs.servers.live_migrate(s, host='hostname',
                                           block_migration=False,
                                           disk_over_commit=False)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': False,
                                               'disk_over_commit': False}})

    def test_live_migrate_server_block_migration_none(self):
        s = self.cs.servers.get(1234)
        ret = s.live_migrate(host='hostname', block_migration=None,
                             disk_over_commit=None)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': False,
                                               'disk_over_commit': False}})

    def test_reset_state(self):
        s = self.cs.servers.get(1234)
        ret = s.reset_state('newstate')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.reset_state(s, 'newstate')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_reset_network(self):
        s = self.cs.servers.get(1234)
        ret = s.reset_network()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.reset_network(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_add_security_group(self):
        s = self.cs.servers.get(1234)
        sg = s.add_security_group('newsg')
        self.assert_request_id(sg, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        sg = self.cs.servers.add_security_group(s, 'newsg')
        self.assert_request_id(sg, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_remove_security_group(self):
        s = self.cs.servers.get(1234)
        ret = s.remove_security_group('oldsg')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.remove_security_group(s, 'oldsg')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_list_security_group(self):
        s = self.cs.servers.get(1234)
        sgs = s.list_security_group()
        self.assert_request_id(sgs, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/1234/os-security-groups')

    def test_evacuate(self):
        s = self.cs.servers.get(1234)
        ret = s.evacuate('fake_target_host', 'True')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        ret = self.cs.servers.evacuate(s, 'fake_target_host',
                                       'False', 'NewAdminPassword')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_interface_list(self):
        s = self.cs.servers.get(1234)
        il = s.interface_list()
        self.assert_request_id(il, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/1234/os-interface')

    def test_interface_list_result_string_representable(self):
        """Test for bugs.launchpad.net/python-novaclient/+bug/1280453."""
        # According to https://github.com/openstack/nova/blob/master/
        # nova/api/openstack/compute/contrib/attach_interfaces.py#L33,
        # the attach_interface extension get method will return a json
        # object partly like this:
        interface_list = [{
            'net_id': 'd7745cf5-63f9-4883-b0ae-983f061e4f23',
            'port_id': 'f35079da-36d5-4513-8ec1-0298d703f70e',
            'mac_addr': 'fa:16:3e:4c:37:c8',
            'port_state': 'ACTIVE',
            'fixed_ips': [
                {
                    'subnet_id': 'f1ad93ad-2967-46ba-b403-e8cbbe65f7fa',
                    'ip_address': '10.2.0.96'
                }]
        }]
        # If server is not string representable, it will raise an exception,
        # because attribute named 'name' cannot be found.
        # Parameter 'loaded' must be True or it will try to get attribute
        # 'id' then fails (lazy load detail), this is exactly same as
        # novaclient.base.Manager._list()
        s = servers.Server(servers.ServerManager, interface_list[0],
                           loaded=True)
        # Trigger the __repr__ magic method
        self.assertEqual('<Server: unknown-name>', '%r' % s)

    def test_interface_attach(self):
        s = self.cs.servers.get(1234)
        ret = s.interface_attach(None, None, None)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/os-interface')
        self.assertIsInstance(ret, servers.NetworkInterface)

    def test_interface_detach(self):
        s = self.cs.servers.get(1234)
        ret = s.interface_detach('port-id')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234/os-interface/port-id')

    def test_create_server_with_description(self):
        self.assertRaises(exceptions.UnsupportedAttribute,
                          self.cs.servers.create,
                          name="My server",
                          description="descr",
                          image=1,
                          flavor=1,
                          meta={'foo': 'bar'},
                          userdata="hello moto",
                          key_name="fakekey"
                          )

    def test_create_server_with_nics_auto(self):
        """Negative test for specifying nics='auto' before 2.37
        """
        self.assertRaises(ValueError,
                          self.cs.servers.create,
                          name='test',
                          image='d9d8d53c-4b4a-4144-a5e5-b30d9f1fe46a',
                          flavor='1',
                          nics='auto')

    def test__validate_create_nics(self):
        if self.cs.api_version > api_versions.APIVersion('2.36'):
            self.assertRaises(ValueError,
                              self.cs.servers._validate_create_nics, None)
        else:
            self.cs.servers._validate_create_nics(None)
            self.assertRaises(ValueError,
                              self.cs.servers._validate_create_nics,
                              mock.Mock())
        self.cs.servers._validate_create_nics(["foo", "bar"])
        self.cs.servers._validate_create_nics(("foo", "bar"))


class ServersV26Test(ServersTest):

    api_version = "2.6"

    def test_get_vnc_console(self):
        s = self.cs.servers.get(1234)
        vc = s.get_vnc_console('novnc')
        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        vc = self.cs.servers.get_vnc_console(s, 'novnc')
        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        # test the case with invalid console type
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_vnc_console,
                          'invalid')

    def test_get_spice_console(self):
        s = self.cs.servers.get(1234)
        sc = s.get_spice_console('spice-html5')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        sc = self.cs.servers.get_spice_console(s, 'spice-html5')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        # test the case with invalid console type
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_spice_console,
                          'invalid')

    def test_get_serial_console(self):
        s = self.cs.servers.get(1234)
        sc = s.get_serial_console('serial')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        sc = self.cs.servers.get_serial_console(s, 'serial')
        self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        # test the case with invalid console type
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_serial_console,
                          'invalid')

    def test_get_rdp_console(self):
        s = self.cs.servers.get(1234)
        rc = s.get_rdp_console('rdp-html5')
        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        rc = self.cs.servers.get_rdp_console(s, 'rdp-html5')
        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        # test the case with invalid console type
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_rdp_console,
                          'invalid')

    def test_get_console_url(self):
        s = self.cs.servers.get(1234)
        vc = s.get_console_url('novnc')
        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        vc = self.cs.servers.get_console_url(s, 'novnc')
        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        # test the case with invalid console type
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_console_url,
                          'invalid')
        # console type webmks is supported since api version 2.8
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_console_url,
                          'webmks')


class ServersV28Test(ServersV26Test):

    api_version = "2.8"

    def test_get_mks_console(self):
        s = self.cs.servers.get(1234)
        mksc = s.get_mks_console()
        self.assert_request_id(mksc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        mksc = self.cs.servers.get_mks_console(s)
        self.assert_request_id(mksc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

    def test_get_console_url(self):
        s = self.cs.servers.get(1234)
        mksc = s.get_console_url('novnc')
        self.assert_request_id(mksc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        mksc = self.cs.servers.get_console_url(s, 'novnc')
        self.assert_request_id(mksc, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/remote-consoles')

        # test the case with invalid console type
        self.assertRaises(exceptions.UnsupportedConsoleType,
                          s.get_console_url,
                          'invalid')


class ServersV214Test(ServersV28Test):

    api_version = "2.14"

    def test_evacuate(self):
        s = self.cs.servers.get(1234)
        s.evacuate('fake_target_host')
        self.assert_called('POST', '/servers/1234/action')
        self.cs.servers.evacuate(s, 'fake_target_host',
                                 password='NewAdminPassword')
        self.assert_called('POST', '/servers/1234/action')


class ServersV217Test(ServersV214Test):

    api_version = "2.17"

    def test_trigger_crash_dump(self):
        s = self.cs.servers.get(1234)
        s.trigger_crash_dump()
        self.assert_called('POST', '/servers/1234/action')
        self.cs.servers.trigger_crash_dump(s)
        self.assert_called('POST', '/servers/1234/action')


class ServersV219Test(ServersV217Test):

    api_version = "2.19"

    def test_create_server_with_description(self):
        self.cs.servers.create(
            name="My server",
            description="descr",
            image=1,
            flavor=1,
            meta={'foo': 'bar'},
            userdata="hello moto",
            key_name="fakekey",
            nics=self._get_server_create_default_nics()
        )
        self.assert_called('POST', '/servers')

    def test_update_server_with_description(self):
        s = self.cs.servers.get(1234)

        s.update(description='hi')
        s.update(name='hi', description='hi')
        self.assert_called('PUT', '/servers/1234')

    def test_rebuild_with_description(self):
        s = self.cs.servers.get(1234)

        ret = s.rebuild(image="1", description="descr")
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')


class ServersV225Test(ServersV219Test):

    api_version = "2.25"

    def test_live_migrate_server(self):
        s = self.cs.servers.get(1234)
        ret = s.live_migrate(host='hostname', block_migration='auto')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto'}})
        ret = self.cs.servers.live_migrate(s, host='hostname',
                                           block_migration='auto')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto'}})

    def test_live_migrate_server_block_migration_true(self):
        s = self.cs.servers.get(1234)
        ret = s.live_migrate(host='hostname', block_migration=True)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': True}})

        ret = self.cs.servers.live_migrate(s, host='hostname',
                                           block_migration=True)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': True}})

    def test_live_migrate_server_block_migration_none(self):
        s = self.cs.servers.get(1234)
        ret = s.live_migrate(host='hostname', block_migration=None)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto'}})

        ret = self.cs.servers.live_migrate(s, host='hostname',
                                           block_migration='auto')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto'}})


class ServersV226Test(ServersV225Test):

    api_version = "2.26"

    def test_tag_list(self):
        s = self.cs.servers.get(1234)
        ret = s.tag_list()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/1234/tags')

    def test_tag_delete(self):
        s = self.cs.servers.get(1234)
        ret = s.delete_tag('tag')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234/tags/tag')

    def test_tag_delete_all(self):
        s = self.cs.servers.get(1234)
        ret = s.delete_all_tags()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/servers/1234/tags')

    def test_tag_add(self):
        s = self.cs.servers.get(1234)
        ret = s.add_tag('tag')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('PUT', '/servers/1234/tags/tag')

    def test_tags_set(self):
        s = self.cs.servers.get(1234)
        ret = s.set_tags(['tag1', 'tag2'])
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('PUT', '/servers/1234/tags')


class ServersV229Test(ServersV226Test):

    api_version = "2.29"

    def test_evacuate(self):
        s = self.cs.servers.get(1234)
        s.evacuate('fake_target_host')
        self.assert_called('POST', '/servers/1234/action',
                           {'evacuate': {'host': 'fake_target_host'}})
        self.cs.servers.evacuate(s, 'fake_target_host', force=True)
        self.assert_called('POST', '/servers/1234/action',
                           {'evacuate': {'host': 'fake_target_host',
                                         'force': True}})


class ServersV230Test(ServersV229Test):

    api_version = "2.30"

    def test_live_migrate_server(self):
        s = self.cs.servers.get(1234)
        ret = s.live_migrate(host='hostname', block_migration='auto')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto'}})
        ret = self.cs.servers.live_migrate(s, host='hostname',
                                           block_migration='auto',
                                           force=True)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto',
                                               'force': True}})


class ServersV232Test(ServersV226Test):

    api_version = "2.32"

    def test_create_server_boot_with_tagged_nics(self):
        nics = [{'net-id': '11111111-1111-1111-1111-111111111111',
                 'tag': 'one'},
                {'net-id': '22222222-2222-2222-2222-222222222222',
                 'tag': 'two'}]
        self.cs.servers.create(name="Server with tagged nics",
                               image=1,
                               flavor=1,
                               nics=nics)
        self.assert_called('POST', '/servers')

    def test_create_server_boot_with_tagged_nics_pre232(self):
        self.cs.api_version = api_versions.APIVersion("2.31")
        nics = [{'net-id': '11111111-1111-1111-1111-111111111111',
                 'tag': 'one'},
                {'net-id': '22222222-2222-2222-2222-222222222222',
                 'tag': 'two'}]
        self.assertRaises(ValueError, self.cs.servers.create,
                          name="Server with tagged nics", image=1, flavor=1,
                          nics=nics)

    def test_create_server_boot_from_volume_tagged_bdm_v2(self):
        bdm = [{"volume_size": "1",
                "volume_id": "11111111-1111-1111-1111-111111111111",
                "delete_on_termination": "0",
                "device_name": "vda", "tag": "foo"}]
        s = self.cs.servers.create(name="My server", image=1, flavor=1,
                                   meta={'foo': 'bar'}, userdata="hello moto",
                                   key_name="fakekey",
                                   block_device_mapping_v2=bdm)
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')

    def test_create_server_boot_from_volume_tagged_bdm_v2_pre232(self):
        self.cs.api_version = api_versions.APIVersion("2.31")
        bdm = [{"volume_size": "1",
                "volume_id": "11111111-1111-1111-1111-111111111111",
                "delete_on_termination": "0",
                "device_name": "vda", "tag": "foo"}]
        self.assertRaises(ValueError, self.cs.servers.create, name="My server",
                          image=1, flavor=1, meta={'foo': 'bar'},
                          userdata="hello moto", key_name="fakekey",
                          block_device_mapping_v2=bdm)


class ServersV2_37Test(ServersV226Test):

    api_version = "2.37"

    def _get_server_create_default_nics(self):
        return 'auto'

    def test_create_server_no_nics(self):
        """Tests that nics are required in microversion 2.37+
        """
        self.assertRaises(ValueError, self.cs.servers.create,
                          name='test',
                          image='d9d8d53c-4b4a-4144-a5e5-b30d9f1fe46a',
                          flavor='1')

    def test_create_server_with_nics_auto(self):
        s = self.cs.servers.create(
            name='test', image='d9d8d53c-4b4a-4144-a5e5-b30d9f1fe46a',
            flavor='1', nics=self._get_server_create_default_nics())
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers')
        self.assertIsInstance(s, servers.Server)


class ServersCreateImageBackupV2_45Test(utils.FixturedTestCase):
    """Tests the 2.45 microversion for createImage and createBackup
    server actions.
    """

    client_fixture_class = client.V1
    data_fixture_class = data.V1
    api_version = '2.45'

    def setUp(self):
        super(ServersCreateImageBackupV2_45Test, self).setUp()
        self.cs.api_version = api_versions.APIVersion(self.api_version)

    def test_create_image(self):
        """Tests the createImage API with the 2.45 microversion which
        does not return the Location header, it returns a json dict in the
        response body with an image_id key.
        """
        s = self.cs.servers.get(1234)
        im = s.create_image('123')
        self.assertEqual('456', im)
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        im = s.create_image('123', {})
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        im = self.cs.servers.create_image(s, '123')
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        im = self.cs.servers.create_image(s, '123', {})
        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')

    def test_backup(self):
        s = self.cs.servers.get(1234)
        # Test backup on the Server object.
        sb = s.backup('back1', 'daily', 1)
        self.assertIn('image_id', sb)
        self.assertEqual('456', sb['image_id'])
        self.assert_request_id(sb, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')
        # Test backup on the ServerManager.
        sb = self.cs.servers.backup(s, 'back1', 'daily', 2)
        self.assertEqual('456', sb['image_id'])
        self.assert_request_id(sb, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action')


class ServersV249Test(ServersV2_37Test):

    api_version = "2.49"

    def test_interface_attach_with_tag(self):
        s = self.cs.servers.get(1234)
        ret = s.interface_attach('7f42712e-63fe-484c-a6df-30ae4867ff66',
                                 None, None, 'test_tag')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST', '/servers/1234/os-interface',
            {'interfaceAttachment':
                {'port_id': '7f42712e-63fe-484c-a6df-30ae4867ff66',
                 'tag': 'test_tag'}})
        self.assertIsInstance(ret, servers.NetworkInterface)

    def test_add_fixed_ip(self):
        # novaclient.v2.servers.Server.add_fixed_ip()
        # is not available after 2.44
        pass

    def test_remove_fixed_ip(self):
        # novaclient.v2.servers.Server.remove_fixed_ip()
        # is not available after 2.44
        pass


class ServersV252Test(ServersV249Test):

    api_version = "2.52"

    def test_create_server_with_tags(self):
        self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            meta={'foo': 'bar'},
            userdata="hello moto",
            key_name="fakekey",
            nics=self._get_server_create_default_nics(),
            tags=['tag1', 'tag2']
        )
        self.assert_called('POST', '/servers',
                           {'server': {
                               'flavorRef': '1',
                               'imageRef': '1',
                               'key_name': 'fakekey',
                               'max_count': 1,
                               'metadata': {'foo': 'bar'},
                               'min_count': 1,
                               'name': 'My server',
                               'networks': 'auto',
                               'tags': ['tag1', 'tag2'],
                               'user_data': 'aGVsbG8gbW90bw=='
                           }}
                           )

    def test_create_server_with_tags_pre_252_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.51')
        self.assertRaises(exceptions.UnsupportedAttribute,
                          self.cs.servers.create,
                          name="My server",
                          image=1,
                          flavor=1,
                          meta={'foo': 'bar'},
                          userdata="hello moto",
                          key_name="fakekey",
                          nics=self._get_server_create_default_nics(),
                          tags=['tag1', 'tag2'])


class ServersV254Test(ServersV252Test):

    api_version = "2.54"

    def test_rebuild_with_key_name(self):
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image="1", key_name="test_keypair")
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rebuild': {
                               'imageRef': '1',
                               'key_name': 'test_keypair'}})

    def test_rebuild_with_key_name_none(self):
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image="1", key_name=None)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rebuild': {
                               'key_name': None,
                               'imageRef': '1'}})

    def test_rebuild_with_key_name_pre_254_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.53')
        ex = self.assertRaises(exceptions.UnsupportedAttribute,
                               self.cs.servers.rebuild,
                               '1234', fakes.FAKE_IMAGE_UUID_1,
                               key_name='test_keypair')
        self.assertIn('key_name', str(ex.message))


class ServersV256Test(ServersV254Test):

    api_version = "2.56"

    def test_migrate_server(self):
        s = self.cs.servers.get(1234)
        ret = s.migrate()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'migrate': {}})
        ret = s.migrate(host='target-host')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'migrate': {'host': 'target-host'}})

    def test_migrate_server_pre_256_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.55')
        s = self.cs.servers.get(1234)
        ex = self.assertRaises(TypeError,
                               s.migrate, host='target-host')
        self.assertIn('host', str(ex))


class ServersV257Test(ServersV256Test):
    """Tests the servers python API bindings with microversion 2.57 where
    personality files are deprecated.
    """
    api_version = "2.57"
    supports_files = False

    def test_create_server_with_files_fails(self):
        ex = self.assertRaises(
            exceptions.UnsupportedAttribute, self.cs.servers.create,
            name="My server", image=1, flavor=1,
            files={
                '/etc/passwd': 'some data',  # a file
                '/tmp/foo.txt': io.StringIO('data'),  # a stream
            }, nics='auto')
        self.assertIn('files', str(ex))

    def test_rebuild_server_name_meta_files(self):
        files = {'/etc/passwd': 'some data'}
        s = self.cs.servers.get(1234)
        ex = self.assertRaises(
            exceptions.UnsupportedAttribute, s.rebuild, image=1, name='new',
            meta={'foo': 'bar'}, files=files)
        self.assertIn('files', str(ex))


class ServersV263Test(ServersV257Test):

    api_version = "2.63"

    def test_create_server_with_trusted_image_certificates(self):
        self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            meta={'foo': 'bar'},
            userdata="hello moto",
            key_name="fakekey",
            nics=self._get_server_create_default_nics(),
            trusted_image_certificates=['id1', 'id2'],
        )
        self.assert_called('POST', '/servers',
                           {'server': {
                               'flavorRef': '1',
                               'imageRef': '1',
                               'key_name': 'fakekey',
                               'max_count': 1,
                               'metadata': {'foo': 'bar'},
                               'min_count': 1,
                               'name': 'My server',
                               'networks': 'auto',
                               'trusted_image_certificates': ['id1', 'id2'],
                               'user_data': 'aGVsbG8gbW90bw=='
                           }}
                           )

    def test_create_server_with_trusted_image_certificates_pre_263_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.62')
        ex = self.assertRaises(
            exceptions.UnsupportedAttribute, self.cs.servers.create,
            name="My server", image=1, flavor=1, meta={'foo': 'bar'},
            userdata="hello moto", key_name="fakekey",
            nics=self._get_server_create_default_nics(),
            trusted_image_certificates=['id1', 'id2'])
        self.assertIn('trusted_image_certificates', str(ex))

    def test_rebuild_server_with_trusted_image_certificates(self):
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image="1", trusted_image_certificates=['id1', 'id2'])
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rebuild': {
                               'imageRef': '1',
                               'trusted_image_certificates': ['id1', 'id2']}})

    def test_rebuild_server_with_trusted_image_certificates_none(self):
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image="1", trusted_image_certificates=None)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'rebuild': {
                               'imageRef': '1',
                               'trusted_image_certificates': None}})

    def test_rebuild_with_trusted_image_certificates_pre_263_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.62')
        ex = self.assertRaises(exceptions.UnsupportedAttribute,
                               self.cs.servers.rebuild,
                               '1234', fakes.FAKE_IMAGE_UUID_1,
                               trusted_image_certificates=['id1', 'id2'])
        self.assertIn('trusted_image_certificates', str(ex))


class ServersV267Test(ServersV263Test):
    """Tests for creating a server with a block_device_mapping_v2 entry
    using volume_type for microversion 2.67.
    """
    api_version = '2.67'

    def test_create_server_boot_from_volume_with_volume_type(self):
        bdm = [{"volume_size": 1,
                "uuid": "11111111-1111-1111-1111-111111111111",
                "delete_on_termination": True,
                "source_type": "snapshot",
                "destination_type": "volume",
                "boot_index": 0,
                "volume_type": "rbd"}]
        s = self.cs.servers.create(
            name="bfv server", image='', flavor=1,
            nics='auto', block_device_mapping_v2=bdm)
        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers', {
            'server': {
                'flavorRef': '1',
                'imageRef': '',
                'name': 'bfv server',
                'networks': 'auto',
                'block_device_mapping_v2': bdm,
                'min_count': 1,
                'max_count': 1,
            }})

    def test_create_server_boot_from_volume_with_volume_type_pre_267(self):
        self.cs.api_version = api_versions.APIVersion('2.66')
        bdm = [{"volume_size": 1,
                "uuid": "11111111-1111-1111-1111-111111111111",
                "delete_on_termination": True,
                "source_type": "snapshot",
                "destination_type": "volume",
                "boot_index": 0,
                "volume_type": "rbd"}]
        ex = self.assertRaises(ValueError, self.cs.servers.create,
                               name="bfv server", image='', flavor=1,
                               nics='none', block_device_mapping_v2=bdm)
        self.assertIn("Block device volume_type is not supported before "
                      "microversion 2.67", str(ex))


class ServersV268Test(ServersV267Test):

    api_version = "2.68"

    def test_evacuate(self):
        s = self.cs.servers.get(1234)
        ret = s.evacuate('fake_target_host')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'evacuate': {'host': 'fake_target_host'}})

        ret = self.cs.servers.evacuate(s, 'fake_target_host')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'evacuate': {'host': 'fake_target_host'}})

        ex = self.assertRaises(TypeError, self.cs.servers.evacuate,
                               'fake_target_host', force=True)
        self.assertIn('force', str(ex))

    def test_live_migrate_server(self):
        s = self.cs.servers.get(1234)
        ret = s.live_migrate(host='hostname', block_migration='auto')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto'}})

        ret = self.cs.servers.live_migrate(s, host='hostname',
                                           block_migration='auto')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'os-migrateLive': {'host': 'hostname',
                                               'block_migration': 'auto'}})

        ex = self.assertRaises(TypeError, self.cs.servers.live_migrate,
                               host='hostname', force=True)
        self.assertIn('force', str(ex))


class ServersV273Test(ServersV268Test):

    api_version = "2.73"

    def test_lock_server(self):
        s = self.cs.servers.get(1234)
        ret = s.lock()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'lock': None})
        ret = s.lock(reason='zombie-apocalypse')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'lock': {'locked_reason': 'zombie-apocalypse'}})

    def test_lock_server_pre_273_fails_with_reason(self):
        self.cs.api_version = api_versions.APIVersion('2.72')
        s = self.cs.servers.get(1234)
        e = self.assertRaises(TypeError,
                              s.lock, reason='blah')
        self.assertIn("unexpected keyword argument 'reason'", str(e))

    def test_filter_servers_unlocked(self):
        # support locked=False
        sl = self.cs.servers.list(search_opts={'locked': False})
        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/servers/detail?locked=False')
        for s in sl:
            self.assertIsInstance(s, servers.Server)


class ServersV274Test(ServersV273Test):

    api_version = "2.74"

    def test_create_server_with_host(self):
        self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            nics="auto",
            host="new-host"
        )
        self.assert_called('POST', '/servers',
                           {'server': {
                               'flavorRef': '1',
                               'imageRef': '1',
                               'max_count': 1,
                               'min_count': 1,
                               'name': 'My server',
                               'networks': 'auto',
                               'host': 'new-host'
                           }}
                           )

    def test_create_server_with_hypervisor_hostname(self):
        self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            nics="auto",
            hypervisor_hostname="new-host"
        )
        self.assert_called('POST', '/servers',
                           {'server': {
                               'flavorRef': '1',
                               'imageRef': '1',
                               'max_count': 1,
                               'min_count': 1,
                               'name': 'My server',
                               'networks': 'auto',
                               'hypervisor_hostname': 'new-host'
                           }}
                           )

    def test_create_server_with_host_and_hypervisor_hostname(self):
        self.cs.servers.create(
            name="My server",
            image=1,
            flavor=1,
            nics="auto",
            host="new-host",
            hypervisor_hostname="new-host"
        )
        self.assert_called('POST', '/servers',
                           {'server': {
                               'flavorRef': '1',
                               'imageRef': '1',
                               'max_count': 1,
                               'min_count': 1,
                               'name': 'My server',
                               'networks': 'auto',
                               'host': 'new-host',
                               'hypervisor_hostname': 'new-host'
                           }}
                           )

    def test_create_server_with_host_pre_274_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.73')
        ex = self.assertRaises(exceptions.UnsupportedAttribute,
                               self.cs.servers.create,
                               name="My server", image=1, flavor=1,
                               nics='auto', host="new-host")
        self.assertIn("'host' argument is only allowed since microversion "
                      "2.74", str(ex))

    def test_create_server_with_hypervisor_hostname_pre_274_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.73')
        ex = self.assertRaises(exceptions.UnsupportedAttribute,
                               self.cs.servers.create,
                               name="My server", image=1, flavor=1,
                               nics='auto', hypervisor_hostname="new-host")
        self.assertIn("'hypervisor_hostname' argument is only allowed since "
                      "microversion 2.74", str(ex))

    def test_create_server_with_host_and_hypervisor_hostname_pre_274_fails(
            self):
        self.cs.api_version = api_versions.APIVersion('2.73')
        ex = self.assertRaises(exceptions.UnsupportedAttribute,
                               self.cs.servers.create,
                               name="My server", image=1, flavor=1,
                               nics='auto', host="new-host",
                               hypervisor_hostname="new-host")
        self.assertIn("'host' argument is only allowed since microversion "
                      "2.74", str(ex))


class ServersV277Test(ServersV274Test):

    api_version = "2.77"

    def test_unshelve(self):
        s = self.cs.servers.get(1234)
        # Test going through the Server object.
        ret = s.unshelve()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': None})
        # Test going through the ServerManager directly.
        ret = self.cs.servers.unshelve(s)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': None})

    def test_unshelve_with_az(self):
        s = self.cs.servers.get(1234)
        # Test going through the Server object.
        ret = s.unshelve(availability_zone='foo-az')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'unshelve': {
                               'availability_zone': 'foo-az'}})
        # Test going through the ServerManager directly.
        ret = self.cs.servers.unshelve(s, availability_zone='foo-az')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/servers/1234/action',
                           {'unshelve': {
                               'availability_zone': 'foo-az'}})

    def test_unshelve_server_pre_277_fails_with_specified_az(self):
        self.cs.api_version = api_versions.APIVersion('2.76')
        s = self.cs.servers.get(1234)
        # Test going through the Server object.
        ex = self.assertRaises(TypeError,
                               s.unshelve,
                               availability_zone='foo-az')
        self.assertIn("unexpected keyword argument 'availability_zone'",
                      str(ex))
        # Test going through the ServerManager directly.
        ex = self.assertRaises(TypeError,
                               self.cs.servers.unshelve,
                               s, availability_zone='foo-az')
        self.assertIn("unexpected keyword argument 'availability_zone'",
                      str(ex))


class ServersV278Test(ServersV277Test):

    api_version = "2.78"

    def test_get_server_topology(self):
        s = self.cs.servers.get(1234)
        topology = s.topology()
        self.assert_request_id(topology, fakes.FAKE_REQUEST_ID_LIST)
        self.assertIsNotNone(topology)
        self.assert_called('GET', '/servers/1234/topology')

        topology_from_manager = self.cs.servers.topology(1234)
        self.assert_request_id(topology, fakes.FAKE_REQUEST_ID_LIST)
        self.assertIsNotNone(topology_from_manager)
        self.assert_called('GET', '/servers/1234/topology')

        self.assertEqual(topology, topology_from_manager)

    def test_get_server_topology_pre278(self):
        self.cs.api_version = api_versions.APIVersion('2.77')
        s = self.cs.servers.get(1234)
        self.assertRaises(exceptions.VersionNotFoundForAPIMethod, s.topology)


class ServersV290Test(ServersV278Test):

    api_version = '2.90'

    def test_create_server_with_hostname(self):
        self.cs.servers.create(
            name='My server',
            image=1,
            flavor=1,
            nics='auto',
            hostname='new-hostname',
        )
        self.assert_called(
            'POST', '/servers',
            {
                'server': {
                    'flavorRef': '1',
                    'imageRef': '1',
                    'max_count': 1,
                    'min_count': 1,
                    'name': 'My server',
                    'networks': 'auto',
                    'hostname': 'new-hostname'
                },
            }
        )

    def test_create_server_with_hostname_pre_290_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.89')
        ex = self.assertRaises(
            exceptions.UnsupportedAttribute,
            self.cs.servers.create,
            name='My server',
            image=1,
            flavor=1,
            nics='auto',
            hostname='new-hostname')
        self.assertIn(
            "'hostname' argument is only allowed since microversion 2.90",
            str(ex))

    def test_rebuild_server_with_hostname(self):
        s = self.cs.servers.get(1234)
        ret = s.rebuild(image="1", hostname='new-hostname')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST', '/servers/1234/action',
            {
                'rebuild': {
                    'imageRef': '1',
                    'hostname': 'new-hostname',
                },
            },
        )

    def test_rebuild_server_with_hostname_pre_290_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.89')
        ex = self.assertRaises(
            exceptions.UnsupportedAttribute,
            self.cs.servers.rebuild,
            '1234', fakes.FAKE_IMAGE_UUID_1,
            hostname='new-hostname')
        self.assertIn('hostname', str(ex))

    def test_update_server_with_hostname(self):
        s = self.cs.servers.get(1234)

        s.update(hostname='new-hostname')
        self.assert_called(
            'PUT', '/servers/1234',
            {
                'server': {
                    'hostname': 'new-hostname',
                },
            },
        )

    def test_update_with_hostname_pre_290_fails(self):
        self.cs.api_version = api_versions.APIVersion('2.89')
        s = self.cs.servers.get(1234)
        ex = self.assertRaises(
            TypeError,
            s.update,
            hostname='new-hostname')
        self.assertIn('hostname', str(ex))


class ServersV291Test(ServersV290Test):

    api_version = "2.91"

    def test_unshelve_with_host(self):
        s = self.cs.servers.get(1234)
        # Test going through the Server object.
        ret = s.unshelve(host='server1')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'host': 'server1'}})
        # Test going through the ServerManager directly.
        ret = self.cs.servers.unshelve(s, host='server1')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'host': 'server1'}})

    def test_unshelve_server_with_az_and_host(self):
        s = self.cs.servers.get(1234)
        # Test going through the Server object.
        ret = s.unshelve(host='server1', availability_zone='foo-az')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'host': 'server1',
                          'availability_zone': 'foo-az'}})
        # Test going through the ServerManager directly.
        ret = self.cs.servers.unshelve(
            s, host='server1', availability_zone='foo-az')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'host': 'server1',
                          'availability_zone': 'foo-az'}})

    def test_unshelve_unpin_az(self):
        s = self.cs.servers.get(1234)
        # Test going through the Server object.
        ret = s.unshelve(availability_zone=None)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'availability_zone': None}})
        # Test going through the ServerManager directly.
        ret = self.cs.servers.unshelve(s, availability_zone=None)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'availability_zone': None}})

    def test_unshelve_server_with_host_and_unpin(self):
        s = self.cs.servers.get(1234)
        # Test going through the Server object.
        ret = s.unshelve(availability_zone=None, host='server1')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'host': 'server1',
                          'availability_zone': None}})
        # Test going through the ServerManager directly.
        ret = self.cs.servers.unshelve(
            s, availability_zone=None, host='server1')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called(
            'POST',
            '/servers/1234/action',
            {'unshelve': {'host': 'server1',
                          'availability_zone': None}})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional.v2.legacy import test_usage


class TestUsageCLI_V240(test_usage.TestUsageCLI):

    COMPUTE_API_VERSION = '2.40'


class TestUsageClient_V240(test_usage.TestUsageClient):

    COMPUTE_API_VERSION = '2.40'

    def test_get(self):
        start, end = self._create_servers_in_time_window()
        tenant_id = self._get_project_id(self.cli_clients.tenant_name)
        usage = self.client.usage.get(
            tenant_id, start=start, end=end, limit=1)
        self.assertEqual(tenant_id, usage.tenant_id)
        self.assertEqual(1, len(usage.server_usages))

    def test_list(self):
        start, end = self._create_servers_in_time_window()
        usages = self.client.usage.list(
            start=start, end=end, detailed=True, limit=1)
        self.assertEqual(1, len(usages))
        self.assertEqual(1, len(usages[0].server_usages))
